@@ -1,5 +1,6 @@
 package WWW::Wikipedia;
 
+use 5.006;
 use strict;
 use warnings;
 use Carp qw( croak );
@@ -8,7 +9,7 @@ use WWW::Wikipedia::Entry;
 
 use base qw( LWP::UserAgent );
 
-our $VERSION = '1.97';
+our $VERSION = '2.05';
 
 use constant WIKIPEDIA_URL =>
     'http://%s.wikipedia.org/w/index.php?title=%s&action=raw';
@@ -78,6 +79,13 @@ you have full access.
 You can turn off the following of wikipedia redirect directives by passing
 a false value to C<follow_redirects>.
 
+Together with the Wiki markup, some entries include HTML tags.
+They can be stripped out using the C<clean_html> option:
+
+   my $wiki = WWW::Wikipedia->new( clean_html => 1 );
+
+See C<clean_html> documentation below for details.
+
 =cut
 
 sub new {
@@ -86,6 +94,7 @@ sub new {
     my $language = delete $opts{ language } || 'en';
     my $follow = delete $opts{ follow_redirects };
     $follow = 1 if !defined $follow;
+    my $clean_html = delete $opts{ clean_html } || 0;
 
     my $self = LWP::UserAgent->new( %opts );
     $self->agent( 'WWW::Wikipedia' );
@@ -93,6 +102,7 @@ sub new {
 
     $self->language( $language );
     $self->follow_redirects( $follow );
+    $self->clean_html( $clean_html );
     $self->parse_head( 0 );
     return $self;
 }
@@ -115,6 +125,26 @@ sub language {
     return $self->{ language };
 }
 
+
+=head2 clean_html()
+
+Allows you to get/set if HTML is being stripped out.
+
+    # set HTML strip
+    $wiki->clean_html( 1 );
+
+This option removes all tags and attributes they might have.
+Their contents, however, is maintained (for now). Comments are
+also removed.
+
+=cut
+
+sub clean_html {
+    my ( $self, $bool ) = @_;
+    $self->{ clean_html } = $bool if defined $bool;
+    return $self->{ clean_html };
+}
+
 =head2 follow_redirects()
 
 By default, wikipeda redirect directives are followed. Set this to false to
@@ -147,17 +177,21 @@ sub search {
     $self->error( undef );
 
     croak( "search() requires you pass in a string" ) if !defined( $string );
-    $string = URI::Escape::uri_escape( $string );
-    my $src = sprintf( WIKIPEDIA_URL, $self->language(), $string );
+    
+    my $enc_string = utf8::is_utf8( $string )
+        ? URI::Escape::uri_escape_utf8( $string )
+        : URI::Escape::uri_escape( $string );
+    my $src = sprintf( WIKIPEDIA_URL, $self->language(), $enc_string );
 
     my $response = $self->get( $src );
     if ( $response->is_success() ) {
-        my $entry = WWW::Wikipedia::Entry->new( $response->content(), $src );
+        my $entry = WWW::Wikipedia::Entry->new( $response->decoded_content(), $src,
+                        clean_html => $self->{ clean_html } );
 
         # look for a wikipedia style redirect and process if necessary
+        # try to catch self-redirects
         return $self->search( $1 )
-            if $self->follow_redirects
-                && $entry->text() =~ /^#REDIRECT (.*)/is;
+            if $self->follow_redirects && $entry->raw() =~ /^#REDIRECT\s*\[\[([^|\]]+)/is && $1 ne $string;
 
         return ( $entry );
     }
@@ -184,7 +218,8 @@ sub random {
         my( $title ) = $response->request->uri =~ m{\.org/wiki/(.+)$};
         $src      = sprintf( WIKIPEDIA_URL, $self->language(), $title );
         $response = $self->get( $src );
-        return WWW::Wikipedia::Entry->new( $response->content(), $src );
+        return WWW::Wikipedia::Entry->new( $response->decoded_content(), $src,
+            clean_html => $self->{ clean_html } );
     }
 
     $self->error( "uhoh, WWW::Wikipedia unable to contact " . $src );
@@ -212,9 +247,16 @@ sub error {
 
 =over 4
 
-=item * Clean up results. Strip HTML.
+=item *
 
-=item * Watch the development of Special:Export XML formatting, eg: http://en.wikipedia.org/wiki/Special:Export/perl
+Be more specific on the HTML clean methodology. For now all tags are removed,
+keeping only their contents. In the future the behaviour might change
+accordingly with each specific tag.
+
+=item *
+
+Watch the development of Special:Export XML formatting, eg:
+http://en.wikipedia.org/wiki/Special:Export/perl
 
 =back
 
@@ -222,10 +264,14 @@ sub error {
 
 =over 4
 
-=item * LWP::UserAgent
+=item * L<LWP::UserAgent>
 
 =back
 
+=head1 REPOSITORY
+
+L<https://github.com/edsu/www-wikipedia>
+
 =head1 AUTHORS
 
 Ed Summers E<lt>ehs@pobox.comE<gt>
@@ -234,7 +280,7 @@ Brian Cassidy E<lt>bricas@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2003-2010 by Ed Summers
+Copyright 2003-2017 by Ed Summers
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
